@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Time::F;
-$Time::F::VERSION = '0.017';
+$Time::F::VERSION = '0.018';
 # ABSTRACT: Formatting times.
 
 use Carp qw/ croak /;
@@ -24,8 +24,8 @@ my %formatter; %formatter = (
     '%d' => fun ($t, $l) { sprintf '%02d', $t->day; },
     '%e' => fun ($t, $l) { sprintf '%2d', $t->day; },
     '%F' => fun ($t, $l) { strftime($t, '%Y-%m-%d', locale => $l); },
-    '%G' => fun ($t, $l) { ((($t->day_of_year < 7) and ($t->week > 1)) ? $t->year - 1 : $t->year); },
-    '%g' => fun ($t, $l) { sprintf ('%02d', ((($t->day_of_year < 7) and ($t->week > 1)) ? substr($t->year - 1, -2) : substr($t->year, -2))); },
+    '%G' => fun ($t, $l) { return $t->clone->day_of_week(4)->year; },
+    '%g' => fun ($t, $l) { sprintf ('%02d', substr($formatter{'%G'}->($t, $l), -2)); },
     '%H' => fun ($t, $l) { sprintf '%02d', $t->hour; },
     '%h' => fun ($t, $l) { $formatter{'%b'}->($t, $l); },
     '%I' => fun ($t, $l) { my $I = $t->hour % 12; sprintf '%02d', $I ? $I : 12; },
@@ -35,7 +35,7 @@ my %formatter; %formatter = (
     '%M' => fun ($t, $l) { sprintf '%02d', $t->minute; },
     '%m' => fun ($t, $l) { sprintf '%02d', $t->month; },
     '%n' => fun ($t, $l) { "\n"; },
-    '%p' => fun ($t, $l) { get_locale(am_pm => $l)->[$t->hour < 12 ? $t->hour < 1 : $t->hour > 12]; },
+    '%p' => fun ($t, $l) { get_locale(am_pm => $l)->[not $t->hour < 12]; },
     '%X' => fun ($t, $l) { strftime($t, get_locale(time => $l), locale => $l); },
     '%x' => fun ($t, $l) { strftime($t, get_locale(date => $l), locale => $l); },
     '%R' => fun ($t, $l) { strftime($t, '%H:%M', locale => $l); },
@@ -44,11 +44,21 @@ my %formatter; %formatter = (
     '%s' => fun ($t, $l) { $t->epoch; },
     '%T' => fun ($t, $l) { strftime($t, '%H:%M:%S', locale => $l); },
     '%t' => fun ($t, $l) { "\t"; },
-    '%U' => fun ($t, $l) { sprintf '%02d', ($t->day_of_week == 7 ? $t->week + 1 : $t->week); },
+    '%U' => fun ($t, $l) {
+        my $t2 = $t->clone->day_of_year(1);
+        $t2->day++ while $t2->day_of_week != 7;
+        if ($t2->day_of_year > $t->day_of_year) { return "00"; }
+        sprintf '%02d', int(($t->day_of_year - $t2->day_of_year) / 7) + 1;
+    },
     '%u' => fun ($t, $l) { $t->day_of_week; },
     '%V' => fun ($t, $l) { sprintf '%02d', $t->week; },
     '%v' => fun ($t, $l) { strftime($t, '%e-%b-%Y', locale => $l); },
-    '%W' => fun ($t, $l) { sprintf '%02d', $t->week; },
+    '%W' => fun ($t, $l) {
+        my $t2 = $t->clone->day_of_year(1);
+        $t2->day++ while $t2->day_of_week != 1;
+        if ($t2->day_of_year > $t->day_of_year) { return "00"; }
+        sprintf '%02d', int(($t->day_of_year - $t2->day_of_year) / 7) + 1;
+    },
     '%w' => fun ($t, $l) { $t->day_of_week == 7 ? 0 : $t->day_of_week; },
     '%Y' => fun ($t, $l) { $t->year; },
     '%y' => fun ($t, $l) { sprintf '%02d', substr $t->year, -2; },
@@ -88,7 +98,7 @@ Time::F - Formatting times.
 
 =head1 VERSION
 
-version 0.017
+version 0.018
 
 =head1 SYNOPSIS
 
