@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Time::P;
-$Time::P::VERSION = '0.017';
+$Time::P::VERSION = '0.018';
 # ABSTRACT: Parse times from strings.
 
 use Carp qw/ croak /;
@@ -180,37 +180,45 @@ fun _parse_struct ($struct, :$locale) {
     my $mday = $struct->{'d'};
     if (not defined $mday) { $mday = $struct->{'e'}; }
 
-    my $s_week = $struct->{'U'};
-    my $m_week = $struct->{'W'} // $struct->{'V'};
+    my $u_week = $struct->{'U'};
+    my $w_week = $struct->{'W'};
+    my $v_week = $struct->{'V'};
+
+    my $wday = $struct->{'u'} // $struct->{'w'};
 
-    my $wday = $struct->{'u'};
-    if (not defined $wday) {
-        $wday = $struct->{'w'};
-        $wday = 7 if defined $wday and $wday == 0;
-    }
     if (not defined $wday) {
         if (defined $struct->{'A'}) {
             $wday = _get_index($struct->{'A'}, @{ get_locale(weekdays => $locale) });
         } elsif (defined $struct->{'a'}) {
             $wday = _get_index($struct->{'a'}, @{ get_locale(weekdays_abbr => $locale) });
         }
-        $wday = 7 if defined $wday and $wday == 0;
+    }
+    $wday = 7 if defined $wday and $wday == 0;
+
+    if (not defined $w_week and defined $u_week) {
+        $w_week = $u_week;
+        if (not defined $wday) { $wday = 7; } # if no wday defined, should set to first day of week, and since the u_week starts at sunday, wday = 7
+        $w_week-- if $wday == 7;
     }
 
-    if (not defined $m_week and defined $s_week and defined $wday) {
-        $m_week = $s_week; $m_week-- if $wday == 7;
+    if (not defined $v_week and defined $w_week) {
+        if ($wyear) { croak "Can't strptime a %G/%g year with a %W/%U week"; }
+
+        my $t = Time::C->new($year // Time::C->now_utc->year);
+        $v_week = $w_week;
+        if (($t->day_of_week > 1) and ($t->day_of_week < 5)) { $v_week++; }
     }
-    if ($wyear and defined $m_week) {
-        $year = Time::C->mktime(year => $year, week => $m_week)->year;
-    } elsif (defined $m_week and $m_week > 1) {
-        if (Time::C->mktime(year => $year, week => $m_week)->year == $year + 1) {
+
+    if ($wyear and defined $v_week and $v_week > 1) {
+        $year = Time::C->mktime(year => $year, week => $v_week)->year;
+    } elsif (defined $v_week and $v_week > 1) {
+        if (Time::C->mktime(year => $year, week => $v_week)->year == $year + 1) {
             $year-- if not defined $month;
         }
     }
 
     # Next try to set up time bits -- these are pretty easy in comparison
 
-    # fix I and l if they're == 12 and it's ... pm? am? gah
     my $hour = $struct->{'H'};
     if (not defined $hour) { $hour = $struct->{'k'}; }
     if (not defined $hour) {
@@ -219,10 +227,9 @@ fun _parse_struct ($struct, :$locale) {
             if (_get_index($struct->{'p'}, @{ get_locale(am_pm => $locale) })) {
                 # PM
                 if ($hour < 12) { $hour += 12; }
-                elsif ($hour == 12) { $hour = 0; }
             } else {
                 # AM
-                if ($hour == 0) { $hour = 12; }
+                if ($hour == 12) { $hour = 0; }
             }
         }
     }
@@ -249,7 +256,7 @@ fun _parse_struct ($struct, :$locale) {
     $struct{hour} = $hour if defined $hour;
     $struct{mday} = $mday if defined $mday;
     $struct{month} = $month if defined $month;
-    $struct{week} = $m_week if defined $m_week;
+    $struct{week} = $v_week if defined $v_week;
     $struct{wday} = $wday if defined $wday;
     $struct{yday} = $yday if defined $yday;
     $struct{year} = $year if defined $year;
@@ -288,7 +295,7 @@ Time::P - Parse times from strings.
 
 =head1 VERSION
 
-version 0.017
+version 0.018
 
 =head1 SYNOPSIS
 
